SuperStrict

Module PUB.MySQL

ModuleInfo "Version:   1.02"
ModuleInfo "Mainwork:  Michael 'Jolinah' Zehr"
ModuleInfo "Author:    Oliver 'Vertex' Skawronek"
ModuleInfo "License:   GPL"

?Win32
	Import "libmysql.a"
?Linux
	Import "-lz"
	Import "libmysqlclient.a"
?
Import BRL.Bank

' MySQL Constants
Const MYSQL_TYPE_DECIMAL     : Int = 1
Const MYSQL_TYPE_TINY        : Int = 2
Const MYSQL_TYPE_SHORT       : Int = 3
Const MYSQL_TYPE_LONG        : Int = 4
Const MYSQL_TYPE_FLOAT       : Int = 5
Const MYSQL_TYPE_DOUBLE      : Int = 6
Const MYSQL_TYPE_NULL        : Int = 7
Const MYSQL_TYPE_TIMESTAMP   : Int = 8
Const MYSQL_TYPE_LONGLONG    : Int = 9
Const MYSQL_TPYE_INT24       : Int = 10
Const MYSQL_TYPE_DATE        : Int = 11
Const MYSQL_TYPE_TIME        : Int = 12
Const MYSQL_TYPE_DATETIME    : Int = 13
Const MYSQL_TYPE_YEAR        : Int = 14
Const MYSQL_TYPE_NEWDATE     : Int = 15
Const MYSQL_TYPE_VARCHAR     : Int = 16
Const MYSQL_TYPE_BIT         : Int = 17
Const MYSQL_TYPE_NEWDECIMAL  : Int = 246
Const MYSQL_TYPE_ENUM        : Int = 247
Const MYSQL_TYPE_SET         : Int = 248
Const MYSQL_TYPE_TINY_BLOB   : Int = 249
Const MYSQL_TYPE_MEDIUM_BLOB : Int = 250
Const MYSQL_TYPE_LONG_BLOB   : Int = 251
Const MYSQL_TYPE_BLOB        : Int = 252
Const MYSQL_TYPE_VAR_STRING  : Int = 253
Const MYSQL_TYPE_STRING      : Int = 254
Const MYSQL_TYPE_GEOMETRY    : Int = 255

' MySQL API
?Win32
	Extern "Win32"
?Linux
	Extern
?
	Function mysql_affected_rows:Int(mysql:Int)
	Function mysql_autocommit:Int(mysql:Int, mode:Int)
	Function mysql_change_user:Int(mysql:Int, user$z, pass$z, db$z)
	Function mysql_close(mysql:Int)
	Function mysql_commit:Int(mysql:Int)
	Function mysql_data_seek(res:Int, offset:Long)
	Function mysql_debug(debug$z)
	Function mysql_dump_debug_info:Int(mysql:Int)
	Function mysql_errno:Int(mysql:Int)
	Function mysql_error$z(mysql:Int)
	Function mysql_fetch_field:Int Ptr(res:Int)
	Function mysql_fetch_field_direct:Int Ptr(res:Int, fieldnr:Int)
	Function mysql_fetch_fields:Int Ptr(res:Int)	
	Function mysql_fetch_lengths:Int Ptr(res:Int)
	Function mysql_fetch_row:Int Ptr(res:Int)
	Function mysql_field_seek:Int(res:Int, offset:Int)
	Function mysql_field_count:Int(mysql:Int)
	Function mysql_field_tell:Int(res:Int)
	Function mysql_free_result(res:Int)
	Function mysql_get_client_info$z()
	Function mysql_get_client_version:Int()
	Function mysql_get_host_info$z(mysql:Int)		
	Function mysql_get_server_version:Int(mysql:Int)
	Function mysql_get_proto_info:Int(mysql:Int)
	Function mysql_get_server_info$z(mysql:Int)
	Function mysql_info$z(mysql:Int)
	Function mysql_init:Int(mysql:Int)
	Function mysql_insert_id:Int(mysql:Int)
	Function mysql_kill:Int(mysql:Int, pid:Int)
	Function mysql_list_dbs:Int(mysql:Int, wild$z)
	Function mysql_list_fields:Int(mysql:Int, table$z, wild$z)
	Function mysql_list_processes:Int(mysql:Int)
	Function mysql_list_tables:Int(mysql:Int, wild$z)
	Function mysql_more_results:Int(mysql:Int)
	Function mysql_next_result:Int(mysql:Int)
	Function mysql_num_fields:Int(res:Int)
	Function mysql_num_rows:Int(res:Int)
	Function mysql_options:Int(mysql:Int, option:Int, arg:Byte Ptr)
	Function mysql_ping:Int(mysql:Int)	
	Function mysql_query:Int(mysql:Int, query$z)
	Function mysql_real_connect:Int(mysql:Int, host$z, user$z, pass$z, db$z, port:Int, ..
	                                unix_socket$z, flag:Int)
	Function mysql_real_escape_string:Int(mysql:Int, to_str:Byte Ptr, from$z, length:Int)
	Function mysql_real_query:Int(mysql:Int, query$z, length:Int)
	Function mysql_refresh:Int(mysql:Int, options:Int)
	Function mysql_reload:Int(mysql:Int)
	Function mysql_rollback:Int(mysql:Int)
	Function mysql_row_seek:Int(res:Int, offset:Int)
	Function mysql_row_tell:Int(res:Int)
	Function mysql_select_db:Int(mysql:Int, db$z)	
	Function mysql_set_server_option:Int(mysql:Int, option:Int)
	Function mysql_sqlstate$z(mysql:Int)
	Function mysql_shutdown:Int(mysql:Int, shut_down_level:Int)
	Function mysql_stat$z(mysql:Int)
	Function mysql_store_result:Int(mysql:Int)
	Function mysql_thread_id:Int(mysql:Int)
	Function mysql_thread_safe:Int()
	Function mysql_use_result:Int(mysql:Int)
	Function mysql_warning_count:Int(mysql:Int)
End Extern

Type TMySQL
	Global State  : Int

	Field MySQL   : Int
	Field Result  : Int

	Function Init()
		If Not TMySQL.State Then TMySQL.State = mysql_init(0)
	End Function

	Function Create:TMySQL(Host:String, User:String, Pass:String, DB:String, Port:Int=0)
		Local MySQL:TMySQL

		If Not TMySQL.State Then TMySQL.Init()

		MySQL = New TMySQL
		MySQL.MySQL = mysql_real_connect(TMySQL.State, Host, User, Pass, DB, Port, Null, 0)
		If MySQL.MySQL = TMySQL.State Then
			Return MySQL
		Else
			Return Null
		EndIf
	End Function

	Method Connect:Int(Host:String, User:String, Pass:String, DB:String, Port:Int = 0)
		If Not TMySQL.State Then TMySQL.Init()
		Self.Close()

		Self.MySQL = mysql_real_connect(TMySQL.State, Host, User, Pass, DB, Port, Null, 0)
		If Self.MySQL = TMySQL.State Then
			Return True
		Else
			Return False
		EndIf
	End Method

	Method Close()
		If Self.MySQL Then
			If Self.Result Then mysql_free_result(Self.Result)
			mysql_close(Self.MySQL)
			Self.MySQL  = 0
			Self.Result = 0
		EndIf
	End Method

	Method Query:TMySQLResult(Query:String)
		Local Result:TMySQLResult, Index:Int, FieldPtr:Int Ptr

		If Not Self.MySQL Then Return Null
		If Self.Result Then mysql_free_result(Self.Result)

		Self.Result = mysql_query(Self.MySQL, Query)

		If Not Self.Result Then
			Self.Result = mysql_store_result(Self.MySQL)
			If Not Self.Result Then
				If mysql_field_count(Self.MySQL) = 0 Then
					Result = New TMySQLResult
					Return Result
				Else
					Return Null
				EndIf
			EndIf

			Result = New TMySQLResult
			Result.ResHandle       = Self.Result
			Result.Columns         = mysql_num_fields(Self.Result)
			Result.Rows            = mysql_num_rows(Self.Result)
			Result.CurrentRow      = Null
			Result.CurrentRowIndex = -1

			If Result.Columns = 0 Then Return Null

			Result.FieldInfo = New TMySQLField[Result.Columns]
			For Index = 0 Until Result.Columns
				FieldPtr = mysql_fetch_field(Self.Result)
				If FieldPtr = Null Then Continue

				Result.FieldInfo[Index] = TMySQLField.GetFromIntPtr(FieldPtr)
			Next

			Self.Result = 0
			Return Result
		Else
			Self.Result = 0
			Return Null
		EndIf
	End Method

	Method AffectedRows:Int()
		If Not Self.MySQL Then Return 0
		Return mysql_affected_rows(Self.MySQL)
	End Method

	Method GetError:String()
		Return mysql_error(Self.MySQL)
	End Method
End Type

Type TMySQLResult
	Field ResHandle : Int
	Field FieldInfo : TMySQLField[]
	Field Columns   : Int
	Field Rows      : Int

	Field CurrentRowIndex : Int
	Field CurrentRow      : Int Ptr

	Method SetCurrentRow:Int(RowIndex:Int)
		If Not Self.ResHandle Then Return False
		If Self.CurrentRowIndex = RowIndex Then Return True
		If RowIndex < 0 Or RowIndex > (Self.Rows - 1) Then Return False

		mysql_data_seek(Self.ResHandle, Long(RowIndex))
		Self.CurrentRowIndex = RowIndex - 1
	End Method

	Method NextRow:Int()
		If Not Self.ResHandle Then Return False

		Self.CurrentRow = mysql_fetch_row(Self.ResHandle)
		Self.CurrentRowIndex :+ 1

		If Self.CurrentRow = Null
			Self.CurrentRowIndex = -1
			Return False
		Else
			Return True
		EndIf
	End Method

	Method GetRow:TMySQLRow(RowIndex:Int)
		Self.SetCurrentRow(RowIndex)
		If NextRow()
			Return TMySQLRow.Create(Self)
		Else
			Return Null
		EndIf
	End Method

	Method Free()
		If Self.ResHandle Then
			mysql_free_result(Self.ResHandle)
			Self.ResHandle = 0
		EndIf
	End Method

	Method Delete()
		Free()
	End Method

	Method ObjectEnumerator:TMySQLResult()
		If Self.CurrentRowIndex > 0 Then Self.SetCurrentRow(0)
		Return Self
	End Method

	Method HasNext:Int()
		Return Self.NextRow()
	End Method

	Method NextObject:Object()
		Return TMySQLRow.Create(Self)
	End Method
End Type

Type TMySQLRow
	Field Result     : TMySQLResult
	Field CurrentRow : Int Ptr

	Function Create:TMySQLRow(Result:TMySQLResult)
		Local Row:TMySQLRow

		Row = New TMySQLRow
		Row.Result     = Result
		Row.CurrentRow = Result.CurrentRow

		Return Row
	End Function

	Method Delete()
		Self.Result = Null
	End Method

	Method GetString:String(ColumnIndex:Int)
		Local Lengths:Int Ptr

		If Not Self.Result.ResHandle Then Return Null
		If Self.CurrentRow = Null Then Return Null
		If ColumnIndex < 0 Or ColumnIndex > (Self.Result.Columns - 1) Then Return Null
		If Not Self.CurrentRow[ColumnIndex] Then Return Null

		Lengths = mysql_fetch_lengths(Self.Result.ResHandle)
		If Lengths Then
			If Lengths[ColumnIndex] = 0 Then Return ""
			Return String.FromBytes(Byte Ptr(CurrentRow[ColumnIndex]), Lengths[ColumnIndex])
		Else
			Return Null
		EndIf
	End Method

	Method GetByteArray:Byte[](ColumnIndex:Int)
		Local Lengths:Int Ptr, Array:Byte[]

		If Not Self.Result.ResHandle Then Return Null
		If Self.CurrentRow = Null Then Return Null
		If ColumnIndex < 0 Or ColumnIndex > (Self.Result.Columns - 1) Then Return Null
		If Not Self.CurrentRow[ColumnIndex] Then Return Null

		Lengths = mysql_fetch_lengths(Self.Result.ResHandle)
		If Lengths Then
			If Lengths[ColumnIndex] = 0 Then Return Null
			Array = New Byte[Lengths[ColumnIndex]]
			MemCopy(Array, Byte Ptr(CurrentRow[ColumnIndex]), Lengths[ColumnIndex])
			Return Array
		Else
			Return Null
		EndIf
	End Method

	Method GetBank:TBank(ColumnIndex:Int)
		Local Lengths:Int Ptr, Bank:TBank

		If Not Self.Result.ResHandle Then Return Null
		If Self.CurrentRow = Null Then Return Null
		If ColumnIndex < 0 Or ColumnIndex > (Self.Result.Columns - 1) Then Return Null
		If Not Self.CurrentRow[ColumnIndex] Then Return Null

		Lengths = mysql_fetch_lengths(Self.Result.ResHandle)
		If Lengths Then
			If Lengths[ColumnIndex] = 0 Then Return Null
			Bank = CreateBank(Lengths[ColumnIndex])
			MemCopy(Bank.Buf(), Byte Ptr(CurrentRow[ColumnIndex]), Lengths[ColumnIndex])
			Return Bank
		Else
			Return Null
		EndIf
	End Method

	Method GetByte:Byte(ColumnIndex:Int)
		Try
			Return Byte(GetString(ColumnIndex))
		Catch Ex:Object
			Return 0
		End Try
	End Method

	Method GetShort:Short(ColumnIndex:Int)
		Try
			Return Short(GetString(ColumnIndex))
		Catch Ex:Object
			Return 0
		End Try
	End Method

	Method GetInt:Int(ColumnIndex:Int)
		Try
			Return Int(GetString(ColumnIndex))
		Catch Ex:Object
			Return 0
		End Try
	End Method

	Method GetLong:Long(ColumnIndex:Int)
		Try
			Return Long(GetString(ColumnIndex))
		Catch Ex:Object
			Return 0
		End Try
	End Method

	Method GetFloat:Float(ColumnIndex:Int)
		Try
			Return Float(GetString(ColumnIndex))
		Catch Ex:Object
			Return 0
		End Try
	End Method

	Method GetDouble:Double(ColumnIndex:Int)
		Try
			Return Double(GetString(ColumnIndex))
		Catch Ex:Object
			Return 0
		End Try
	End Method
End Type

Type TMySQLField
	Field Name         : String
	Field OrgName      : String
	Field Table        : String
	Field OrgTable     : String
	Field DB           : String
	Field Catalog      : String
	Field DefaultValue : String
	Field Length       : Int
	Field Flags        : Int
	Field Decimals     : Int
	Field CharsetNr    : Int
	Field FieldType    : Int

	Function GetFromIntPtr:TMySQLField(FieldPtr:Int Ptr)
		Local MField:TMySQLField

		If FieldPtr = Null Then Return Null

		MField = New TMySQLField
		If FieldPtr[ 0] Then MField.Name         = String.FromCString(Byte Ptr(FieldPtr[0]))
		If FieldPtr[ 1] Then MField.OrgName      = String.FromCString(Byte Ptr(FieldPtr[1]))
		If FieldPtr[ 2] Then MField.Table        = String.FromCString(Byte Ptr(FieldPtr[2]))
		If FieldPtr[ 3] Then MField.OrgTable     = String.FromCString(Byte Ptr(FieldPtr[3]))
		If FieldPtr[ 4] Then MField.DB           = String.FromCString(Byte Ptr(FieldPtr[4]))
		If FieldPtr[ 5] Then MField.Catalog      = String.FromCString(Byte Ptr(FieldPtr[5]))
		If FieldPtr[ 6] Then MField.DefaultValue = String.FromCString(Byte Ptr(FieldPtr[6]))
		MField.Length    = FieldPtr[ 7]
		MField.Flags     = FieldPtr[16]
		MField.Decimals  = FieldPtr[17]
		MField.CharsetNr = FieldPtr[18]
		MField.FieldType = FieldPtr[19]

		Return MField																							
	End Function
End Type